"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const spec_1 = require("@hayspec/spec");
const __1 = require("../..");
const spec = new spec_1.Spec();
spec.test('method `handle` handles an error and returns error codes', (ctx) => __awaiter(this, void 0, void 0, function* () {
    const handler = new __1.Handler();
    const recipes = [
        { code: 200, handler: (e) => e.message === 'foo' },
        { code: 201, handler: (e) => e.message === 'bar' },
        { code: 400, handler: (e) => e.message === 'foo' },
        { code: 401, handler: (e) => e.message === 'bar' },
    ];
    const error = new Error('foo');
    const codes = yield handler.handle(error, recipes);
    ctx.deepEqual(codes, [200, 400]);
}));
spec.test('method `handle` can fail fast', (ctx) => __awaiter(this, void 0, void 0, function* () {
    const handler = new __1.Handler({
        failFast: true,
    });
    const recipes = [
        { code: 400, handler: (v) => true },
        { code: 401, handler: (v) => true },
        { code: 402, handler: (v) => true },
    ];
    const codes = yield handler.handle('foo', recipes);
    ctx.deepEqual(codes, [400]);
}));
spec.test('method `handle` supports conditional handlers', (ctx) => __awaiter(this, void 0, void 0, function* () {
    const handler = new __1.Handler();
    const recipes = [
        { code: 100, handler: (v) => true, condition: (v) => true },
        { code: 200, handler: (v) => true, condition: (v) => false },
        { code: 300, handler: (v) => true },
    ];
    const codes = yield handler.handle(true, recipes);
    ctx.deepEqual(codes, [100, 300]);
}));
spec.test('method `handle` passes context to each handler', (ctx) => __awaiter(this, void 0, void 0, function* () {
    const handler = new __1.Handler({
        context: { foo: 'foo' },
    });
    const recipes = [
        { code: 100, handler(v) { return v === this.foo; } },
    ];
    const codes = yield handler.handle('foo', recipes);
    ctx.deepEqual(codes, [100]);
}));
spec.test('method `handle` passes context to each condition', (ctx) => __awaiter(this, void 0, void 0, function* () {
    const handler = new __1.Handler({
        context: { foo: 'foo' },
    });
    const recipes = [
        { code: 100, handler: (v) => true, condition(v) { return v === this.foo; } },
    ];
    const codes = yield handler.handle('foo', recipes);
    ctx.deepEqual(codes, [100]);
}));
spec.test('method `handle` passes recipe to each handler', (ctx) => __awaiter(this, void 0, void 0, function* () {
    const handler = new __1.Handler();
    const recipes = [
        { code: 100, handler: (v, r) => v === r.code },
    ];
    const codes = yield handler.handle(100, recipes);
    ctx.deepEqual(codes, [100]);
}));
spec.test('method `handle` passes recipe to each condition', (ctx) => __awaiter(this, void 0, void 0, function* () {
    const handler = new __1.Handler();
    const recipes = [
        { code: 100, handler: (v) => true, condition: (v, r) => r.code === 100 },
    ];
    const codes = yield handler.handle(true, recipes);
    ctx.deepEqual(codes, [100]);
}));
exports.default = spec;
//# sourceMappingURL=handler.test.js.map